/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "c3type.h"
#include "c3ext.h"
/******************************************************************************/
/*             Allocate memory for the transmit filter variables.
*******************************************************************************/
Tx_Flt_Mem_Alloc()
{
 void *tmp;
 void *calloc();
 tmp=calloc(TX_FLT_SIZE,sizeof(double));
 Ck_Calloc(tmp);
 Tx_Flt=(double *)tmp;
 tmp=calloc(TX_FLT_SIZE,sizeof(double));
 Ck_Calloc(tmp);
 Tx_Reg=(double *)tmp;
 return(0);
}
/******************************************************************************/
/*                Allocate memory for correlator variables.
*******************************************************************************/
int Corr_Mem_Alloc()
{
 Corr_Flt_Alloc();
 Corr_Reg_Alloc();
 Corr_Avg_Alloc();
 return(0);
}
/******************************************************************************/
/*          Allocate memory for storing correlator taps.
*******************************************************************************/
int Corr_Flt_Alloc()
{
 void *tmp;
 void *calloc();
 tmp=calloc(CORR_SIZE,sizeof(double));
 Ck_Calloc(tmp);
 Corr_C_Flt=(double *)tmp;
 tmp=calloc(CORR_SIZE,sizeof(double));
 Ck_Calloc(tmp);
 Corr_S_Flt=(double *)tmp;
 return(0);
}
/******************************************************************************/
/*              Allocate memory for storing correlator input.
*******************************************************************************/
int Corr_Reg_Alloc()
{
 void *tmp;
 void *calloc();
 tmp=calloc(CORR_SIZE,sizeof(double));
 Ck_Calloc(tmp);
 Corr_C_Reg=(double *)tmp;
 tmp=calloc(CORR_SIZE,sizeof(double));
 Ck_Calloc(tmp);
 Corr_S_Reg=(double *)tmp;
 return(0);
}
/******************************************************************************/
/*     Allocate memory for storing the average of the correlator output.
*******************************************************************************/
int Corr_Avg_Alloc()
{
 void *tmp;
 void *calloc();
 tmp=calloc(CORR_SIZE,sizeof(double));
 Ck_Calloc(tmp);
 Corr_C_Avg=(double *)tmp;
 tmp=calloc(CORR_SIZE,sizeof(double));
 Ck_Calloc(tmp);
 Corr_S_Avg=(double *)tmp;
 return(0);
}
/******************************************************************************/
/*                    Allocate memory for the trellis.
*******************************************************************************/
int Trel_Mem_Alloc()
{
 Nxt_State_Alloc();
 M_C_Alloc();
 M_S_Alloc();
 return(0);
}
/******************************************************************************/
/*               Allocate memory for storing next states.
*******************************************************************************/
int Nxt_State_Alloc()
{
 int cnt;
 void *tmp;
 void *calloc();
 tmp=calloc(Num_States,sizeof(N_St_Array));
 Ck_Calloc(tmp);
 N_St_Array=tmp;
 for(cnt=ZERO;cnt<Num_States;cnt++)
 {
  tmp=calloc(CONSTELL_SIZE,sizeof(int));
  Ck_Calloc(tmp);
  N_St_Array[cnt]=(int *)tmp;
 }
 return(0);
}
/******************************************************************************/
/* Allocate memory for storing the cosine of the message phase contributions
   corresponding to distinct sampling phases.
*******************************************************************************/
int M_C_Alloc()
{
 int cnt1,cnt2;
 void *tmp;
 void *calloc();
 for(cnt1=ZERO;cnt1<BB_SAMP;cnt1++)
 {
  tmp=calloc(Num_States,sizeof(M_C_Array[cnt1]));
  Ck_Calloc(tmp);
  M_C_Array[cnt1]=tmp;
  for(cnt2=ZERO;cnt2<Num_States;cnt2++)
  {
   tmp=calloc(CONSTELL_SIZE,sizeof(double));
   Ck_Calloc(tmp);
   M_C_Array[cnt1][cnt2]=(double *)tmp;
  }
 }
 return(0);
}
/******************************************************************************/
/* Allocate memory for storing the sine of the message phase contributions
   corresponding to distinct sampling phases.
*******************************************************************************/
int M_S_Alloc()
{
 int cnt1,cnt2;
 void *tmp;
 void *calloc();
 for(cnt1=ZERO;cnt1<BB_SAMP;cnt1++)
 {
  tmp=calloc(Num_States,sizeof(M_S_Array[cnt1]));
  Ck_Calloc(tmp);
  M_S_Array[cnt1]=tmp;
  for(cnt2=ZERO;cnt2<Num_States;cnt2++)
  {
   tmp=calloc(CONSTELL_SIZE,sizeof(double));
   Ck_Calloc(tmp);
   M_S_Array[cnt1][cnt2]=(double *)tmp;
  }
 }
 return(0);
}
/******************************************************************************/
/*                 Check for memory allocation failure.
*******************************************************************************/
int Ck_Calloc(tmp)
void *tmp;
{
 if(tmp==NULL)
  exit(1);
 return(0);
}
/******************************************************************************/
