/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "c3type.h"
#include "c3ext.h"
/******************************************************************************/
/*                       Preprocessor functions.
*******************************************************************************/
int Pre_Process()
{
 Check_Input();
 Open_Files();
 Init_Var();
 Get_Tx_Flt();
 Get_Tx_Int_Flt();
 Get_Corr();
 Get_LPF();
 Init_Sim();
 return(0);
}
/******************************************************************************/
/*                       Check the input parameters.
*******************************************************************************/
int Check_Input()
{
 double curr_mod_fac;
 if(TX_FLT_SIZE%TX_FLT_SAMP)
  exit(8);
 if(LPF_DELAY%TWO)
  exit(7);
 curr_mod_fac=TWICEPI*MOD_INX/((double)TX_FLT_SAMP*BIT_RATE);
 if(((MOD_FAC-curr_mod_fac)>ERROR) || ((MOD_FAC-curr_mod_fac)< -ERROR))
  exit(9);
 if(TX_INT_SIZE%PB_SAMP)
  exit(14);
 if((TX_INT_SIZE*TX_FLT_SAMP/PB_SAMP) != TX_FLT_SIZE)
  exit(15);
 return(0);
}
/******************************************************************************/
/*                        Open files for graphics.
*******************************************************************************/
int Open_Files()
{
 fp1=fopen("c3data/txflt.dat","w");
 fp2=fopen("c3data/lpf.dat","w");
 fp3=fopen("c3data/corc.dat","w");
 fp4=fopen("c3data/cors.dat","w");
 fp5=fopen("c3data/bbac.dat","w");
 fp6=fopen("c3data/gflt.dat","w");
 fp7=fopen("c3data/txint.dat","w");
 return(0);
}
/******************************************************************************/
/*                          Initialize variables.
*******************************************************************************/
int Init_Var()
{
 F_s_PB=(double)PB_SAMP*BIT_RATE;
 F_s_BB=(double)BB_SAMP*BIT_RATE;
 F_s_Tx_Flt=(double)TX_FLT_SAMP*BIT_RATE;
 B_t=0.25*F_s_PB;
 printf("\t Passband sampling frequency is %f kHz.\n\n",F_s_PB);
 printf("\t Baseband sampling frequency is %f kHz.\n\n",F_s_BB);
 printf("\t Transmit filter sampling frequency is %f kHz.\n\n",F_s_Tx_Flt);
 printf("\t Decimation factor is %d.\n\n",D_FAC);
 printf("\t 3-dB bandwidth for RC is %f.\n\n",B_t);
 return(0);
}
/******************************************************************************/
/*              Get the transmit-filter (Gaussian) response.
*******************************************************************************/
int Get_Tx_Flt()
{
 Tx_Flt_Mem_Alloc();
 Get_Gauss_Flt();
 Print_Gauss_Flt();
 Conv_Gauss_Rect();
 Normalize_Txflt();
 Print_Tx_Flt();
 return(0);
}
/******************************************************************************/
/*                        Print the Gaussian filter.
*******************************************************************************/
int Print_Gauss_Flt()
{
 int cnt;
 double sum;
 sum=0.0;
 for(cnt=ZERO;cnt<G_FLT_SIZE;cnt++)
 {
  sum=sum+Tx_Flt[cnt];
  fprintf(fp6,"%lf %lf\n",(double)cnt,Tx_Flt[cnt]);
 }
 sum=sum/F_s_Tx_Flt;
 printf("\t Sum of Gaussian filter taps is %16.10lf.\n\n",sum);
 return(0);
}
/******************************************************************************/
/*   Convolve Gaussian filter with a rectangular pulse of one bit duration.
*******************************************************************************/
int Conv_Gauss_Rect()
{
 int cnt;
 double allone[TX_FLT_SAMP],op[TX_FLT_SIZE];
 for(cnt=ZERO;cnt<TX_FLT_SAMP;cnt++)
  allone[cnt]=1.0;
 Convolve(allone,Tx_Flt,op,TX_FLT_SAMP,G_FLT_SIZE);
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  Tx_Flt[cnt]=op[cnt];
 return(0);
}
/******************************************************************************/
/*         Normalize the coefficients so that the sum equals 0.5.
*******************************************************************************/
int Normalize_Txflt()
{
 int cnt;
 double sum;
 sum=0.0;
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  sum=sum+Tx_Flt[cnt];
 sum=sum/F_s_Tx_Flt;
 printf("\t Actual area under the Gaussian pulse is %f.\n\n",sum);
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  Tx_Flt[cnt]=Tx_Flt[cnt]*0.5/sum;
 return(0);
}
/******************************************************************************/
/*           Print the transmit-filter response obtained as above.
*******************************************************************************/
int Print_Tx_Flt()
{
 int cnt;
 double sum;
 sum=0.0;
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
 {
  fprintf(fp1,"%f %f\n",(double)cnt,Tx_Flt[cnt]);
  sum=sum+Tx_Flt[cnt];
 }
 sum=sum/F_s_Tx_Flt;
 printf("\t Normalized area under the Gaussian pulse is %f.\n\n",sum);
 return(0);
}
/******************************************************************************/
/*                     Get the modified transmit filter.
*******************************************************************************/
int Get_Tx_Int_Flt()
{
 Get_Int_Flt_Taps();
 Print_Int_Flt();
 return(0);
}
/******************************************************************************/
/*                  Get the modified transmit filter taps.
*******************************************************************************/
int Get_Int_Flt_Taps()
{
 int cnt,ulimit,inx;
 Samp_Phase=ZERO;
 if(Samp_Phase >= (TX_FLT_SAMP/PB_SAMP))
  exit(12);
 printf("\t Sampling phase chosen is %d.\n\n",Samp_Phase);
 for(cnt=ZERO;cnt<TX_INT_SIZE;cnt++)
 {
  ulimit=Samp_Phase+cnt*TX_FLT_SAMP/PB_SAMP;
  if(ulimit >= TX_FLT_SIZE)
   exit(16);
  Tx_Int_Flt[cnt]=0.0;
  for(inx=ZERO;inx<=ulimit;inx++)
   Tx_Int_Flt[cnt]=Tx_Int_Flt[cnt]+MOD_FAC*Tx_Flt[inx];
 } 
 return(0);
}
/******************************************************************************/
/*                  Print the modified transmit filter taps.
*******************************************************************************/
int Print_Int_Flt()
{
 int cnt;
 for(cnt=ZERO;cnt<TX_INT_SIZE;cnt++)
  fprintf(fp7,"%lf %lf\n",(double)cnt,Tx_Int_Flt[cnt]);
 return(0);
}
/******************************************************************************/
/*                 Get the correlator impulse response.
*******************************************************************************/
int Get_Corr()
{
 int tmpsym;
 Corr_Mem_Alloc();
 Init_Corr_Tx_Flt();
 Tx_Tot_Phase=0.0;
 Corr_Flt_Inx=CORR_SIZE-1;
 for(Symcnt=ZERO;Symcnt<TWO;Symcnt++)
 {
  tmpsym=Symval;                                  /* Store current input bit. */
  for(Ipcnt=ZERO;Ipcnt<TX_FLT_SAMP;Ipcnt++)
   Get_Next_Corr_Tap();
  Symval= -tmpsym;                                    /* Invert the next bit. */
 }
 Print_Corr_Flt();
 return(0);
}
/******************************************************************************/
/*     Initialize the transmit filter memory to a steady state condition.
*******************************************************************************/
int Init_Corr_Tx_Flt()
{
 int cnt;
 Symval=ONE;
 for(cnt=(TX_FLT_SIZE-1);cnt>=ZERO;cnt--)
 {
  if((cnt%TX_FLT_SAMP) == (TX_FLT_SAMP-1))
  {
   Tx_Reg[cnt]=(double)Symval;
   Symval= -Symval;                                   /* Invert the next bit. */
  }
  else
   Tx_Reg[cnt]=0.0;
 }
 return(0);
}
/******************************************************************************/
/*                     Get the next correlation filter tap.
*******************************************************************************/
int Get_Next_Corr_Tap()
{
 int cnt;
 Right_Shift(Tx_Reg,TX_FLT_SIZE);
 Tx_Reg[ZERO]=(double)Symval;
 Tx_Op=0.0;
 for(cnt=Ipcnt;cnt<TX_FLT_SIZE;cnt=cnt+TX_FLT_SAMP)
  Tx_Op=Tx_Op+Tx_Flt[cnt]*Tx_Reg[cnt];
 Tx_Tot_Phase=Tx_Tot_Phase+Tx_Op*MOD_FAC;
 if(Corr_Flt_Inx < ZERO)
  exit(10);
 Corr_C_Flt[Corr_Flt_Inx]=cos(Tx_Tot_Phase);
 Corr_S_Flt[Corr_Flt_Inx]=sin(Tx_Tot_Phase);
 Corr_Flt_Inx--;
 Symval=ZERO;
 return(0);
}
/******************************************************************************/
/*                    Print the correlation filter taps.
*******************************************************************************/
int Print_Corr_Flt()
{
 int cnt;
 for(cnt=ZERO;cnt<CORR_SIZE;cnt++)
 {
  fprintf(fp3,"%f %f\n",(double)cnt,Corr_C_Flt[cnt]);
  fprintf(fp4,"%f %f\n",(double)cnt,Corr_S_Flt[cnt]);
 }
 return(0);
}
/******************************************************************************/
/*               Get the lowpass (interpolating) filter taps.
*******************************************************************************/
int Get_LPF()
{
 int cnt;
 double delay;
 delay=(double)LPF_DELAY;
 Get_SRRC(Lpf_Tap,LPF_SIZE,delay);
 Normalize_SRRC();
 /*Ck_SRRC();*/
 for(cnt=ZERO;cnt<LPF_SIZE;cnt++)
  fprintf(fp2,"%f %f\n",(double)cnt,Lpf_Tap[cnt]);
 return(0);
}
/******************************************************************************/
/*              Normalize the SRRC taps to have a gain of 2.
*******************************************************************************/
int Normalize_SRRC()
{
 int cnt;
 double sum,newsum;
 sum=newsum=0.0;
 for(cnt=ZERO;cnt<LPF_SIZE;cnt++)
  sum=sum+Lpf_Tap[cnt];
 printf("\t Actual gain of LPF at dc is %f.\n\n",sum);
 for(cnt=ZERO;cnt<LPF_SIZE;cnt++)
 {
  Lpf_Tap[cnt]=2.0*Lpf_Tap[cnt]/sum;
  newsum=newsum+Lpf_Tap[cnt];
 }
 printf("\t Normalized gain of LPF at dc is %f.\n\n",newsum);
 return(0);
}
/******************************************************************************/
/*               Check if the LPF has an SRRC response.
*******************************************************************************/
int Ck_SRRC()
{
 int cnt;
 double op[2*LPF_SIZE-1];
 Convolve(Lpf_Tap,Lpf_Tap,op,LPF_SIZE,LPF_SIZE);
 for(cnt=ZERO;cnt<(2*LPF_SIZE-1);cnt++)
  printf("%d %f\n",cnt,op[cnt]);
 return(0);
}
/******************************************************************************/
