/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "c3type.h"
#include "c3ext.h"
/******************************************************************************/
/*                            Postprocessor.
*******************************************************************************/
int Viterbi()
{
 if(Depth<(VA_DEPTH-ONE))
  Pre_VA_Depth();
 else if(Depth==(VA_DEPTH-ONE))
 {
  VA_Link_Alloc();
  VA_Sub_Alloc();
  Metric_Reinit();
 }
 if(Depth==VA_DEPTH)
  Post_VA_Depth();
 return(0);
}
/******************************************************************************/
/*              Procedures for depth less than 'VA_DEPTH-1'.
*******************************************************************************/
int Pre_VA_Depth()
{
 VA_Link_Alloc();
 VA_Sub_Alloc();
 Metric_Reinit();
 Calculate_Metric();
 return(0);
}
/******************************************************************************/
/* Do memory allocation for a link. The 'fstdepth' corresponds to the
   first incoming symbol in time and the 'lstdepth' corresponds to the 
   last incoming symbol in time.
*******************************************************************************/
int VA_Link_Alloc()
{
 void *tmp;
 void *malloc();
 struct trellis *curr;
 tmp=malloc(sizeof(struct trellis));
 Ck_Calloc(tmp);
 curr=(struct trellis *)tmp;
 if(Depth==ZERO)
  fstdepth=curr;
 curr->prvdepth=lstdepth;
 curr->nxtdepth=NULL;
 if(lstdepth!=NULL)
  lstdepth->nxtdepth=curr;
 lstdepth=curr;                             /* This completes double linking. */
 return(0);
}
/******************************************************************************/
/*          Do memory allocations for arrays within a structure.
*******************************************************************************/
int VA_Sub_Alloc()
{
 void *tmp;
 void *calloc();
 tmp=calloc(Num_States,sizeof(int));          /* For storing previous states. */
 Ck_Calloc(tmp);
 lstdepth->stpath=(int *)tmp;
 tmp=calloc(Num_States,sizeof(int));     /* For storing corresponding inputs. */
 Ck_Calloc(tmp);
 lstdepth->ippath=(int *)tmp;
 tmp=calloc(Num_States,sizeof(double));           /* For storing Re. metrics. */
 Ck_Calloc(tmp);
 lstdepth->VA_Re_Wght=(double *)tmp;
 Depth++;
 return(0);
}
/******************************************************************************/
/*                       Processing at 'VA_DEPTH'.
*******************************************************************************/
int Post_VA_Depth()
{
 Calculate_Metric();
 Get_Max_Metric();
 Get_Last_Symbol();
 if(NORMALIZE)
  Normalize_Metric();
 Linked_List_Reinit();
 Metric_Reinit();
 return(0);
}
/******************************************************************************/
/* Calculate the metric associated with each state for the current time
   instant.
*******************************************************************************/
int Calculate_Metric()
{
 if(lstdepth==NULL)
  exit(3);
 for(P_State=ZERO;P_State<Num_States;P_State++)
 {
  for(P_Input=ZERO;P_Input<CONSTELL_SIZE;P_Input++)
  {
   N_State=N_St_Array[P_State][P_Input];
   Get_Metric();
   Get_Survivor();
  }
 }
 return(0);
}
/******************************************************************************/
/*                           Compute the metric.
*******************************************************************************/
int Get_Metric()
{
 int ifac;
 double reip,imip,tcos,tsin,reop;
 New_Re_Wght=0.0;
 for(ifac=ZERO;ifac<BB_SAMP;ifac++)
 {
  reip=VA_C_Ip[ifac];
  imip=VA_S_Ip[ifac];
  tcos=M_C_Array[ifac][P_State][P_Input];
  tsin=M_S_Array[ifac][P_State][P_Input];
  reop=reip*tcos+imip*tsin;
  New_Re_Wght=New_Re_Wght+FAC*reop;
 }
 return(0);
}
/******************************************************************************/
/*               Update the survivor to the next state.
*******************************************************************************/
int Get_Survivor()
{
 Get_New_Wght();
 Get_Existing_Wght();
 Get_Net_Wght();
 if((Old_Wght < THRESH_WGHT) || (New_Wght > Old_Wght))
  Survivor_Info();
 return(0);
}
/******************************************************************************/
/*            Get the prospective new weight at the next depth.
*******************************************************************************/
int Get_New_Wght()
{
 struct trellis *prev;
 prev=lstdepth->prvdepth;
 if(prev!=NULL)
  New_Re_Wght=prev->VA_Re_Wght[P_State]+New_Re_Wght;
 return(0);
}
/******************************************************************************/
/*                 Get the existing weight at the next state.
*******************************************************************************/
int Get_Existing_Wght()
{
 Old_Re_Wght=lstdepth->VA_Re_Wght[N_State];
 return(0);
}
/******************************************************************************/
/*                  Get the net existing and new weights.
*******************************************************************************/
int Get_Net_Wght()
{
 Old_Wght=Old_Re_Wght;
 New_Wght=New_Re_Wght;
 if(New_Wght < THRESH_WGHT)
  exit(4);
 return(0);
}
/******************************************************************************/
/*                    Get survivor for the next state.
*******************************************************************************/
int Survivor_Info()
{
 lstdepth->VA_Re_Wght[N_State]=New_Re_Wght;
 lstdepth->stpath[N_State]=P_State;
 lstdepth->ippath[N_State]=P_Input;
 /*printf("PS:%d inp:%d ",P_State,P_Input);
 printf("NS:%d new re:%f ",N_State,lstdepth->VA_Re_Wght[N_State]);*/
 return(0);
}
/******************************************************************************/
