/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
/*                        FILE POINTER VARIABLES.
*******************************************************************************/
FILE *fp1,*fp2,*fp3,*fp4,*fp5,*fp6,*fp7;                    /* File pointers. */
/******************************************************************************/
/*                        ERROR COUNTER VARIABLE.
*******************************************************************************/
long Err_Cnt;                                 /* Counts the number of errors. */
long Probe_Cnt;                                        /* No. of bits probed. */
long Probe_Err_Cnt;                           /* No. of probed bits in error. */
/******************************************************************************/
/*                           COUNTER VARIABLES.
*******************************************************************************/
long Symcnt;                                                 /* Symbol count. */
long V_Sym_Cnt;                                 /* Valid symbols transmitted. */
int Ipcnt;                                            /* Interpolation index. */
/******************************************************************************/
/*              VARIABLES RELATED TO THE SAMPLING FREQUENCY.
*******************************************************************************/
double F_s_Tx_Flt;              /* Sampling frequency of the transmit filter. */
double F_s_BB;                  /* Sampling frequency of the baseband signal. */
double F_s_PB;                  /* Sampling frequency of the passband signal. */
/******************************************************************************/
/*       VARIABLES RELATED TO THE BUFFER CONTAINING THE SYNC PATTERN.
*******************************************************************************/
int Event;                                    /* 1:Sync obtained;0:otherwise. */
int Sync_Inx;                                           /* Sync buffer index. */
int Sync_Buf[SYNC_SIZE];                                      /* Sync buffer. */
int Rx_Sync_Buf[SYNC_SIZE];                        /* Buffer at the receiver. */
/******************************************************************************/
/*                VARIABLES RELATED TO THE ERROR BUFFER.
*******************************************************************************/
int Err_Buf_Inx;                                       /* Error buffer index. */
int Err_Buf[ERR_BUF_SIZE];                                   /* Error buffer. */
/******************************************************************************/
/*               VARIABLES RELATED TO THE TRANSMIT-FILTER.
*******************************************************************************/
int Symval;                                     /* Symbols at Tx. flt. input. */
int Tx_D_Fac;                           /* Decimation factor at Tx. Flt. o/p. */
int Samp_Phase;                    /* Sampling phase chosen for transmission. */
double Tx_Op;                                       /* Output of the Tx. flt. */
double *Tx_Flt;                                             /* Tx. flt. taps. */
double *Tx_Reg;                                           /* Tx. flt. memory. */
double Tx_Int_Flt[TX_INT_SIZE];                    /* Modified Tx. flt. taps. */
double Tx_Int_Reg[TX_INT_SIZE];                  /* Modified Tx. flt. memory. */
/******************************************************************************/
/*                     VARIABLES RELATED TO THE MODULATOR.
*******************************************************************************/
double M_Phase;                                  /* Phase due to the message. */
double Base_Phase;                         /* Phase state at the transmitter. */
double Car_Phase;                                /* Phase due to the carrier. */
double Tx_Tot_Phase;                                          /* Total phase. */
double Pb_Sig;                                  /* Sample of passband signal. */
/******************************************************************************/
/*            VARIABLES RELATED TO THE GAUSSIAN NOISE GENERATOR.
*******************************************************************************/
unsigned short n1store[3];                              /* Stores PNS status. */
unsigned short n2store[3];                              /* Stores PNS status. */
double nvals[NVALSIZE];                          /* Contains U & G rand. var. */
/******************************************************************************/
/*                VARIABLES RELATED TO THE RECEIVER CARRIER.
*******************************************************************************/
int My_Cos;                                  /* Cosine of the carrier (PI/2). */
int My_Sin;                                    /* Sine of the carrier (PI/2). */
int Delta_Cos;                       /* Used to update cosine of the carrier. */
int Delta_Sin;                         /* Used to update sine of the carrier. */
double Rx_C_Op;                                       /* Cosine mixer output. */
double Rx_S_Op;                                         /* Sine mixer output. */
/******************************************************************************/
/*                VARIABLES RELATED TO THE LOWPASS FILTER.
*******************************************************************************/
double B_t;                                          /* 3-dB of the RC pulse. */
double Lpf_Tap[LPF_SIZE];                                        /* LPF taps. */
double Lpf_Reg[LPF_SIZE];                                      /* LPF memory. */
double Delay_Reg[DELAY_REG_SIZE];                          /* Delay register. */
double Rx_C_BB;                                           /* Output of delay. */
double Rx_S_BB;                                             /* Output of LPF. */
/******************************************************************************/
/*                 VARIABLES RELATED TO THE CORRELATION FILTER.
*******************************************************************************/
int Tim_Acq;                                /* 1:Timing acquired;0:otherwise. */
int Tim_Inx;                                  /* Index corresponding to peak. */
int Fst_Samp;                /* Corresponds to lowest tap indices of Tx. Flt. */
int Corr_Avg_Inx;                                /* Index to averaging array. */
int Corr_Flt_Inx;                                /* Correlation filter index. */
double Corr_C_Op;                                   /* Re. correlator output. */
double Corr_S_Op;                                   /* Im. correlator output. */
double *Corr_C_Flt;                           /* Re. correlation filter taps. */
double *Corr_S_Flt;                           /* Im. correlation filter taps. */
double *Corr_C_Reg;                                /* Re. correlation memory. */
double *Corr_S_Reg;                                /* Im. correlation memory. */
double *Corr_C_Avg;                      /* Re. average of correlator output. */
double *Corr_S_Avg;                      /* Im. average of correlator output. */
/******************************************************************************/
/*                VARIABLES RELATED TO THE ALL "-1" SEQUENCE.
*******************************************************************************/
int F_off_Reg_Inx;                                 /* Index to the registers. */
double Cos_off;                            /* Cosine of current offset phase. */
double Sin_off;                              /* Sine of current offset phase. */
double F_off_C_Op;                           /* Cosine part of canceller o/p. */
double F_off_S_Op;                             /* Sine part of canceller o/p. */
double F_off_C_Reg[F_OFF_S_PERIOD];              /* Store cosine of 1 period. */
double F_off_S_Reg[F_OFF_S_PERIOD];                /* Store sine of 1 period. */
/******************************************************************************/
/*      VARIABLES RELATED TO STORING SAMPLES THAT ARE INPUT TO THE VA.
*******************************************************************************/
int BB_Ipcnt;                         /* Interpolation count in the baseband. */
double VA_C_Ip[BB_SAMP];                          /* Cosine part of VA input. */
double VA_S_Ip[BB_SAMP];                            /* Sine part of VA input. */
/******************************************************************************/
/*                    VARIABLES RELATED TO THE TRELLIS.
*******************************************************************************/
int Mem_Size;                               /* Memory of the transmit-filter. */
int Num_States;                     /* Number of states in the super-trellis. */
int P_State;                                 /* Present state in the trellis. */
int N_State;                                    /* Next state in the trellis. */
int P_Input;                                                /* Present input. */
int **N_St_Array;                                    /* Array of next states. */
double **M_C_Array[BB_SAMP];                     /* Cosine of message phases. */
double **M_S_Array[BB_SAMP];                       /* Sine of message phases. */
/******************************************************************************/
/*               VARIABLES RELATED TO THE VITERBI DECODER.
*******************************************************************************/
int Depth;                              /* Gives the current depth of the VA. */
int VA_Op_Sym;                               /* Symbol number released by VA. */
int Max_Wght_St;                 /* State associated with the maximum metric. */
int Min_Wght_St;                 /* State associated with the minimum metric. */
double Theta;                       /* Angle of complex metric at each state. */
double Max_Wght;                                           /* Maximum weight. */
double Min_Wght;                                           /* Minimum weight. */
double Cur_Wght;                                           /* Current weight. */
double New_Re_Wght;                          /* Re. new weight at next state. */
double New_Im_Wght;                          /* Im. new weight at next state. */
double New_Wght;                                           /* Net new weight. */
double Old_Re_Wght;                          /* Re. old weight at next state. */
double Old_Im_Wght;                          /* Im. old weight at next state. */
double Old_Wght;                                           /* Net old weight. */
struct trellis                                       /* Structure of trellis. */
{
 int *stpath;                                    /* Surviving path of states. */
 int *ippath;                                    /* Surviving path of inputs. */
 double *VA_Re_Wght;                /* Re. metric associated with each state. */
 double *VA_Im_Wght;                /* Im. metric associated with each state. */
 struct trellis *prvdepth;                      /* Address of previous depth. */
 struct trellis *nxtdepth;                          /* Address of next depth. */
};
struct trellis *fstdepth,*lstdepth;                             /* Templates. */
/******************************************************************************/
