/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "c5type.h"
#include "c5ext.h"
/******************************************************************************/
/*                              Simuation.
*******************************************************************************/
int Simulation()
{
 for(Symcnt=ZERO;Symcnt<MAXSYM;Symcnt++)
 {
  /*printf("sym:%d\n",Symcnt);*/
  Get_Sym();
  for(Ipcnt=ZERO;Ipcnt<PB_SAMP;Ipcnt++)
  {
   Get_Txflt_Op();
   Modulate();
   Add_Noise();
   Receive();
  }
  Adj_Base_Phase();
 }
 return(0);
}
/******************************************************************************/
/*                      Get the transmit-filter output.
*******************************************************************************/
int Get_Txflt_Op()
{
 int cnt;
 Right_Shift(Tx_Int_Reg,TX_INT_SIZE);
 Tx_Int_Reg[ZERO]=(double)Symval;
 Tx_Op=0.0;
 for(cnt=Ipcnt;cnt<TX_INT_SIZE;cnt=cnt+PB_SAMP)
  Tx_Op=Tx_Op+Tx_Int_Flt[cnt]*Tx_Int_Reg[cnt];
 Symval=ZERO;
 return(0);
}
/******************************************************************************/
/*                   Modulate the baseband pulse.
*******************************************************************************/
int Modulate()
{
 Tx_Tot_Phase=Base_Phase+Tx_Op+Car_Phase;
 Car_Phase=Car_Phase+PIBY2;
 if(Car_Phase > TWICEPI)
  Car_Phase=Car_Phase-TWICEPI;
 else if(Car_Phase < -TWICEPI)
  Car_Phase=Car_Phase+TWICEPI;
 Pb_Sig=cos(Tx_Tot_Phase);
 /*printf("ipcnt:%d tx tot phase:%lf\n",Ipcnt,Tx_Tot_Phase);*/
 /*printf("%lf\n",Pb_Sig);*/
 return(0);
}
/******************************************************************************/
/*                        Add white Gaussian noise.
*******************************************************************************/
int Add_Noise()
{
 double erand48();
 nvals[URADIUS]=erand48(n1store);
 nvals[UTHETA]=erand48(n2store);
 My_Gauss(VARIANCE,nvals);
 if(Event)                                            /* @Uncomment this. */
  Pb_Sig=Pb_Sig+nvals[GRAND];
 return(0);
}
/******************************************************************************/
/*                        Input to the receiver.
*******************************************************************************/
int Receive()
{
 Get_Mixer_Op();
 if(!My_Sin)                      /* Input corresponding to largest tap is 0. */
 {
  /*printf("sym:%d ipcnt:%d bbip:%d\n",Symcnt,Ipcnt,BB_Ipcnt);*/
  Get_Rx_BB();
  if(!Tim_Acq)
   Acquire_Timing();
  if(Symcnt >= VIT_START)
   Input_To_Viterbi();
  Update_Ctrs();
 }
 Update_LO();
 return(0);
}
/******************************************************************************/
/*          Multiply using carrier at PI/2 and shift to LPF & delay.
*******************************************************************************/
int Get_Mixer_Op()
{
 Rx_C_Op=Pb_Sig*(double)My_Cos;
 Rx_S_Op=Pb_Sig*(double)My_Sin;
 Right_Shift(Delay_Reg,DELAY_REG_SIZE);
 Right_Shift(Lpf_Reg,LPF_SIZE);
 Delay_Reg[ZERO]=Rx_C_Op;
 Lpf_Reg[ZERO]=Rx_S_Op;
 return(0);
}
/******************************************************************************/
/*                      Get the complex baseband. 
*******************************************************************************/
int Get_Rx_BB()
{
 int cnt;
 Rx_S_BB=0.0;
 for(cnt=ONE;cnt<LPF_SIZE;cnt=cnt+TWO)
  Rx_S_BB=Rx_S_BB+Lpf_Tap[cnt]*Lpf_Reg[cnt];
 Rx_C_BB=Delay_Reg[DELAY_REG_SIZE-ONE];
 return(0);
}
/******************************************************************************/
/*          Input to the Viterbi Algorithm if condition is met.
*******************************************************************************/
int Input_To_Viterbi()
{
 F_Off_Cancel();
 Store_BB_Samp();
 return(0);
}
/******************************************************************************/
/*           Subtract the phase due to the all "-1" bit sequence.
*******************************************************************************/
int F_Off_Cancel()
{
 Cos_off=F_off_C_Reg[F_off_Reg_Inx];
 Sin_off=F_off_S_Reg[F_off_Reg_Inx];
 F_off_C_Op=Rx_C_BB*Cos_off+Rx_S_BB*Sin_off;
 F_off_S_Op=Rx_S_BB*Cos_off-Rx_C_BB*Sin_off;
 return(0);
}
/******************************************************************************/
/*               Store the current complex baseband sample.
*******************************************************************************/
int Store_BB_Samp()
{
 int index;
 index=BB_Ipcnt-Fst_Samp;
 if(index < ZERO)
  index=BB_SAMP+index;
 VA_C_Ip[index]=F_off_C_Op;
 VA_S_Ip[index]=F_off_S_Op;
 if(index == (BB_SAMP-ONE))
  Proc_VA_Op();
 /*printf("index:%d bbip:%d foffinx:%d\n",index,BB_Ipcnt,F_off_Reg_Inx);*/
 /*printf("%lf\n",Rx_C_BB*Rx_C_BB+Rx_S_BB*Rx_S_BB);*/
 /*printf("%lf %lf\n",F_off_C_Op,F_off_S_Op);*/
 return(0);
}
/******************************************************************************/
/*             Feed inputs to VA and process the VA outputs.
*******************************************************************************/
int Proc_VA_Op()
{
 Viterbi();
 if(!Event)
  Get_Sync();
 if(Event)
  Err_Cmp();
 return(0);
}
/******************************************************************************/
/*                          Get the sync pattern.
*******************************************************************************/
int Get_Sync()
{
 int cnt,found;
 for(cnt=SYNC_SIZE-1;cnt>ZERO;cnt--)
  Rx_Sync_Buf[cnt]=Rx_Sync_Buf[cnt-ONE];
 Rx_Sync_Buf[ZERO]=VA_Op_Sym;
 found=TRUE;
 for(cnt=ZERO;(cnt<SYNC_SIZE) && (found);cnt++)
 {
  if(Rx_Sync_Buf[cnt] != Sync_Buf[cnt])
   found=FALSE;
 }
 /*for(cnt=ZERO;cnt<SYNC_SIZE;cnt++)
  printf("cnt:%d syn:%d rxsyn:%d\n",cnt,Sync_Buf[cnt],Rx_Sync_Buf[cnt]);*/
 Event=found;
 if(Event)
  printf("\t Sync obtained. Symbol count is %d.\n\n",Symcnt);
 return(0);
}
/******************************************************************************/
/*          Compare the transmitted and received symbols for error.
*******************************************************************************/
int Err_Cmp()
{
 int cnt;
 if(Symcnt >= TRN_PERIOD)
 {
  V_Sym_Cnt++;
  if(VA_Op_Sym != Err_Buf[SYNC_SIZE-ONE])
   Err_Cnt++;
  Get_Probe_Error();
 }
 for(cnt=ZERO;cnt<Err_Buf_Inx-ONE;cnt++)
  Err_Buf[cnt]=Err_Buf[cnt+ONE];
 Err_Buf_Inx--;
 if(Err_Buf_Inx < ZERO)
  exit(6);
 return(0);
}
/******************************************************************************/
/*                  Check for error at the probing instant.
*******************************************************************************/
int Get_Probe_Error()
{
 int modsym;
 modsym=Symcnt%NORM_RATE;
 if((modsym >= PROBE_L_LIM) && (modsym <= PROBE_U_LIM))
 {
  Probe_Cnt++;
  if(VA_Op_Sym != Err_Buf[SYNC_SIZE-ONE])
   Probe_Err_Cnt++;
 }
 return(0);
}
/******************************************************************************/
/*                   Update the baseband sample counter.
*******************************************************************************/
int Update_Ctrs()
{
 BB_Ipcnt++;
 if(BB_Ipcnt == BB_SAMP)
  BB_Ipcnt=ZERO;
 F_off_Reg_Inx++;
 if(F_off_Reg_Inx == F_OFF_S_PERIOD)
  F_off_Reg_Inx=ZERO;
 return(0);
}
/******************************************************************************/
/*                          Adjust the base phase.
*******************************************************************************/
int Adj_Base_Phase()
{
 Base_Phase=Base_Phase+Tx_Int_Reg[TX_INT_SIZE-ONE]*PIBY2;
 if(Base_Phase > TWICEPI)
  Base_Phase=Base_Phase-TWICEPI;
 else if(Base_Phase < -TWICEPI)
  Base_Phase=Base_Phase+TWICEPI;
 /*printf("tx reg ip:%lf bphase:%lf\n",Tx_Int_Reg[TX_INT_SIZE-1],Base_Phase);*/
 return(0);
}
/******************************************************************************/
