/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "c5type.h"
#include "c5ext.h"
/******************************************************************************/
/*                             Acquire timing.
*******************************************************************************/
int Acquire_Timing()
{
 Shift_Into_Corr();
 if((Symcnt > TIM_ACQ_TIME) && (Corr_Avg_Inx == ZERO))
 {
  Get_Tim_Info();
  Proc_Tim_Info();
  Set_Up_Trellis();
  Get_F_Off();
  Print_Corr_Op();
  Free_Mem();
 }
 return(0);
}
/******************************************************************************/
/* Get the correlator output. Note that the interpolation factor is
   "2*Tx_D_Fac".
*******************************************************************************/
int Shift_Into_Corr()
{
 for(Corr_Flt_Inx=ZERO;Corr_Flt_Inx<(TWO*Tx_D_Fac);Corr_Flt_Inx++)
 {
  /*printf("ipcnt:%d corripcnt:%d ",Ipcnt,Corr_Flt_Inx);
  printf("bbcnt:%d corravginx:%d\n",BB_Ipcnt,Corr_Avg_Inx);*/
  Right_Shift(Corr_C_Reg,CORR_SIZE);
  Right_Shift(Corr_S_Reg,CORR_SIZE);
  Corr_C_Reg[ZERO]=Rx_C_BB;
  Corr_S_Reg[ZERO]=Rx_S_BB;
  Get_Corr_Op();
  Rx_C_BB=Rx_S_BB=0.0;              /* Interpolate by a factor of "Tx_D_Fac". */
 }
 return(0);
}
/******************************************************************************/
/*               Get the complex output of the correlator.
*******************************************************************************/
int Get_Corr_Op()
{
 int cnt;
 Corr_C_Op=Corr_S_Op=0.0;
 for(cnt=Corr_Flt_Inx;cnt<CORR_SIZE;cnt=cnt+TWO*Tx_D_Fac)
 {
  Corr_C_Op=Corr_C_Op+Corr_C_Flt[cnt]*Corr_C_Reg[cnt];
  Corr_C_Op=Corr_C_Op+Corr_S_Flt[cnt]*Corr_S_Reg[cnt];
  Corr_S_Op=Corr_S_Op+Corr_C_Flt[cnt]*Corr_S_Reg[cnt];
  Corr_S_Op=Corr_S_Op-Corr_S_Flt[cnt]*Corr_C_Reg[cnt];
 }
 /*printf("%f\n",Corr_C_Op*Corr_C_Op+Corr_S_Op*Corr_S_Op);*/
 if(Symcnt > FREE_WHEEL)
  Update_Avg_Op();
 return(0);
}
/******************************************************************************/
/*          Update the average of the complex correlator outputs.
*******************************************************************************/
int Update_Avg_Op()
{
 Corr_C_Avg[Corr_Avg_Inx]=Corr_C_Avg[Corr_Avg_Inx]+Corr_C_Op;
 Corr_S_Avg[Corr_Avg_Inx]=Corr_S_Avg[Corr_Avg_Inx]+Corr_S_Op;
 Corr_Avg_Inx++;
 if(Corr_Avg_Inx == CORR_SIZE)
  Corr_Avg_Inx=ZERO;
 return(0);
}
/******************************************************************************/
/* Get the timing information. Outputs at all time instants must be averaged
   same number of times.
*******************************************************************************/
int Get_Tim_Info()
{
 int cnt;
 double max,cmax,cavg,savg;
 max=Corr_C_Avg[0]*Corr_C_Avg[0]+Corr_S_Avg[0]*Corr_S_Avg[0];
 Tim_Inx=ZERO;
 for(cnt=ONE;cnt<CORR_SIZE;cnt++)
 {
  cmax=Corr_C_Avg[cnt]*Corr_C_Avg[cnt]+Corr_S_Avg[cnt]*Corr_S_Avg[cnt];
  if(cmax > max)
  {
   max=cmax;
   Tim_Inx=cnt;
  }
 }
 return(0);
}
/******************************************************************************/
/*                        Process the timing information.
*******************************************************************************/
int Proc_Tim_Info()
{
 printf("\t Correlation peak occurred at index %d.\n\n",Tim_Inx);
 Fst_Samp=(BB_Ipcnt+ONE+Tim_Inx/(TWO*Tx_D_Fac))%BB_SAMP;
 Fst_Samp=Fst_Samp-(CORR_SIZE/(TWO*Tx_D_Fac)-ONE)%BB_SAMP;
 if(Fst_Samp < ZERO)
  Fst_Samp=BB_SAMP+Fst_Samp;
 Tim_Inx=TWO*Tx_D_Fac-ONE-Tim_Inx%(TWO*Tx_D_Fac);
 Tim_Acq=TRUE;
 printf("\t Lowest index of transmit filter used is %d.\n\n",Tim_Inx);
 printf("\t First sample in the group is %d.\n\n",Fst_Samp);
 return(0);
}
/******************************************************************************/
/*                     Print the correlator output.
*******************************************************************************/
int Print_Corr_Op()
{
 int cnt,inx;
 double mag;
 for(cnt=ZERO;cnt<5*CORR_SIZE;cnt++)
 {
  inx=cnt%CORR_SIZE;
  mag=Corr_C_Avg[inx]*Corr_C_Avg[inx]+Corr_S_Avg[inx]*Corr_S_Avg[inx];
  mag=mag/100.0;
  fprintf(fp5,"%lf %lf\n",(double)cnt,mag);
 }
 return(0);
}
/******************************************************************************/
