/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
/******************************************************************************/
/*                        LENGTH OF SIMULATION RUN.
*******************************************************************************/
#define SEG1 100                                  /* 1st segment of training. */
#define SEG2 100                                  /* 2nd segment of training. */
#define TRN_PERIOD (SEG1+SEG2)              /* Length of the training period. */
#define MAXSYM (1000000+TRN_PERIOD)  /* Number of symbols used in simulation. */
#define FREE_WHEEL 50                    /* Do nothing for this many symbols. */
#define TIM_ACQ_TIME 60            /* Acquire timing after this many symbols. */
#define VIT_START 70                        /* Start the VA after 70 symbols. */
/******************************************************************************/
/*                     GENERAL SIMULATION PARAMETERS.
*******************************************************************************/
#define PB_SAMP 4                         /* Samples per bit in the passband. */
#define BB_SAMP 2                         /* Samples per bit in the baseband. */
#define TX_FLT_SAMP 96                /* Samples per bit for transmit filter. */
#define D_FAC (PB_SAMP/BB_SAMP)                         /* Decimation factor. */
#define BIT_RATE 1.152                                           /* Bit-rate. */
#define BW_3_DB (0.5*BIT_RATE)       /* 3-dB bandwidth of the Gaussian pulse. */
#define MOD_INX 0.5                                      /* Modulation index. */
#define CONSTELL_SIZE 2                                 /* Binary signalling. */
#define NORMALIZE 0                       /* 1:Normalize metrics;0:otherwise. */
#define NORM_RATE 100                                  /* Normalization rate. */
#define PROBE_L_LIM 91        /* To probe percent error due to normalization. */
#define PROBE_U_LIM 95        /* To probe percent error due to normalization. */
/******************************************************************************/
/*              BIT SEQUENCE SYNC AND ERROR BUFFER PARAMETERS.
*******************************************************************************/
#define SYNC_SIZE 16                             /* Size of the sync pattern. */
#define ERR_BUF_SIZE 64                          /* Size of the error buffer. */
/******************************************************************************/
/*                      TRANSMIT FILTER PARAMETERS.
*******************************************************************************/
#define G_FLT_SIZE 193                           /* Order of Gaussian filter. */
#define TX_FLT_SIZE (G_FLT_SIZE+TX_FLT_SAMP-1)           /* Order of Tx. flt. */
#define TX_INT_SIZE 12                   /* Size of modified transmit filter. */
/******************************************************************************/
/*                     CORRELATION FILTER PARAMETERS.
*******************************************************************************/
#define CORR_SIZE (2*TX_FLT_SAMP)                 /* Order of the correlator. */
/******************************************************************************/
/*                  GAUSSIAN NOISE GENERATOR PARAMETERS.
*******************************************************************************/
#define VARIANCE 0.600                                     /* Noise variance. */
#define NVALSIZE 3
#define URADIUS 0
#define UTHETA 1
#define GRAND 2
/******************************************************************************/
/*                LOWPASS (INTERPOLATING) FILTER PARAMETERS.
*******************************************************************************/
#define ROLL_OFF 0.161                                    /* Roll-off factor. */
#define LPF_DELAY 24                          /* Delay of the LPF in samples. */
#define LPF_SIZE (2*LPF_DELAY+1)                      /* Order of the filter. */
#define DELAY_REG_SIZE (LPF_DELAY+1)          /* Order of the delay register. */
/******************************************************************************/
/*                  ALL MINUSONE SEQUENCE PARAMETERS.
*******************************************************************************/
#define F_OFF_B_PERIOD 4                              /* Periodicity in bits. */
#define F_OFF_S_PERIOD (BB_SAMP*F_OFF_B_PERIOD)    /* Periodicity in samples. */
/******************************************************************************/
/*                 PARAMETERS RELATED TO THE VITERBI.
*******************************************************************************/
#define VA_DEPTH 4                                        /* Depth of the VA. */
#define FAC 0.0001                          /* 1 for norm;0.0001 for no norm. */
#define FAC_THRESH 0.01                  /* Threshold for multiplying factor. */
#define THRESH_WGHT (-1000.0)                          /* Coherent threshold. */
#define NC_TH_WGHT (1000000.0)                     /* Non-coherent threshold. */
/******************************************************************************/
/*                             CONSTANTS.
*******************************************************************************/
#define BINSIZE 16         /* Size of array to store states in binary format. */
#define TWO 2
#define ONE 1
#define ZERO 0
#define TRUE 1
#define FALSE 0
#define LEFT 0
#define RIGHT 1
#define MINUSONE (-1)
#define ERROR 0.000001
#define FRAC_ZERO 0.0
#define MY_PI 3.14159265358979323846
#define PIBY2 1.57079632679489661923
#define TWICEPI 6.28318530717958647692
/*#define MOD_FAC 0.05681410325502374924*/    /* 2*PI*MOD_INX/F_s_Tx_Flt;48R. */
#define MOD_FAC 0.02840705162751187462                            /* For 96R. */
/******************************************************************************/
