/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "c5type.h"
#include "c5ext.h"
/******************************************************************************/
/*                        Get the Gaussian filter.
*******************************************************************************/
int Get_Gauss_Flt()
{
 int cnt;
 double x,xsq,texpo,delay,index,lnsqrt2,gain;
 lnsqrt2=log(sqrt(2.0));
 gain=MY_PI/lnsqrt2;
 gain=BW_3_DB*sqrt(gain);
 delay=((double)(G_FLT_SIZE-1))*0.5;
 index=0.0;
 for(cnt=ZERO;cnt<G_FLT_SIZE;cnt++)
 {
  x=BW_3_DB*MY_PI*(index-delay)/F_s_Tx_Flt;
  xsq= -x*x/lnsqrt2;
  texpo=exp(xsq);
  Tx_Flt[cnt]=gain*texpo;
  index=index+1.0;
 }
 return(0);
}
/******************************************************************************/
/*   Shifts the contents of the register to the right to allow a new sample.
*******************************************************************************/
int Right_Shift(treg,REGSIZE)
double treg[];
int REGSIZE;
{
 int cnt;
 for(cnt=(REGSIZE-1);cnt>ZERO;cnt--)
  treg[cnt]=treg[cnt-1];
 return(0);
}
/******************************************************************************/
/*                 Update the sine and cosine of the LO.
*******************************************************************************/
int Update_LO()
{
 if(My_Cos == MINUSONE)
  Delta_Cos=ONE;
 else if(My_Cos == ONE)
  Delta_Cos= MINUSONE;
 else if(My_Sin == MINUSONE)
  Delta_Sin=ONE;
 else if(My_Sin == ONE)
  Delta_Sin= MINUSONE;
 My_Cos=My_Cos+Delta_Cos;
 My_Sin=My_Sin+Delta_Sin;
 return(0);
}
/******************************************************************************/
/* Input arrays: ip1[losize] is smaller array.ip2[hisize] is larger array.
   Output array: op[].
*******************************************************************************/
int Convolve(ip1,ip2,op,losize,hisize)
int losize,hisize;
double ip1[],ip2[],op[];
{
 int opcnt,ipcnt,loop;
 for(opcnt=ZERO;opcnt<(losize+hisize-1);opcnt++)
 {
  op[opcnt]=0.0;
  ipcnt=ZERO;
  loop=TRUE;
  while((ipcnt<losize)&&(loop==TRUE))
  {
   if((opcnt-ipcnt)<ZERO)
    loop=FALSE;
   else if((opcnt-ipcnt)>=hisize)
    ipcnt=opcnt-hisize+1;
   else
   {
    op[opcnt]=op[opcnt]+ip1[ipcnt]*ip2[opcnt-ipcnt];
    ipcnt++;
   }
  }
 }
 return(0);
}
/******************************************************************************/
/*               Get Square Root of Raised Cosine impulse response.
*******************************************************************************/
int Get_SRRC(filter,FILTERSIZE,delta)
double filter[],delta;
int FILTERSIZE;
{
 int cnt;
 double n,denom,x,theta1,theta2,term1,term2;
 x=MY_PI*sqrt(2.0*B_t);
 n=0.0;
 for(cnt=ZERO;cnt<FILTERSIZE;cnt++)
 {
  theta1=2.0*MY_PI*B_t*(1.0+ROLL_OFF)*(n-delta)/F_s_PB;
  theta2=2.0*MY_PI*B_t*(1.0-ROLL_OFF)*(n-delta)/F_s_PB;
  term1=8.0*B_t*ROLL_OFF*cos(theta1);
  if(((n-delta)<ERROR)&&((delta-n)<ERROR))
   term2=2.0*MY_PI*B_t*(1.0-ROLL_OFF);
  else
   term2=F_s_PB*sin(theta2)/(n-delta);
  denom=8.0*B_t*ROLL_OFF*(n-delta)/F_s_PB;
  denom=denom*denom;
  denom=x*(1.0-denom);
  if((denom<ERROR)&&((0.0-denom)<ERROR))
   exit(11);
  else
   filter[cnt]=(term1+term2)/denom;
  n=n+1.0;
 }
 return(0);
}
/******************************************************************************/
/*       Convert uniform random variables to gaussian random variables.
*******************************************************************************/
int My_Gauss(tvar,tnarry)
double tvar,tnarry[];
{
 double r,theta;
 r= -2.0*tvar*log(1.0-tnarry[URADIUS]);
 if(r<0.0)
 {
  printf("r:%f URADIUS:%f\n",r,tnarry[URADIUS]);
  r=0.0;
 }
 r=sqrt(r);
 theta=2.0*MY_PI*tnarry[UTHETA];
 tnarry[GRAND]=r*cos(theta);
 return(0);
}
/******************************************************************************/
