/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "c5type.h"
#include "c5ext.h"
/******************************************************************************/
/*           Get the maximum metric and the associated state.
*******************************************************************************/
int Get_Max_Metric()
{
 int cnt;
 Max_Wght_St=ZERO;
 Max_Wght=(lstdepth->VA_Re_Wght[ZERO])*(lstdepth->VA_Re_Wght[ZERO]);
 Max_Wght=Max_Wght+(lstdepth->VA_Im_Wght[ZERO])*(lstdepth->VA_Im_Wght[ZERO]);
 for(cnt=ONE;cnt<Num_States;cnt++)
 {
  Cur_Wght=(lstdepth->VA_Re_Wght[cnt])*(lstdepth->VA_Re_Wght[cnt]);
  Cur_Wght=Cur_Wght+(lstdepth->VA_Im_Wght[cnt])*(lstdepth->VA_Im_Wght[cnt]);
  if(Cur_Wght > Max_Wght)
  {
   Max_Wght=Cur_Wght;
   Max_Wght_St=cnt;
  }
 }
 /*Print_Wghts();*/
 return(0);
}
/******************************************************************************/
/*                          Print the weights.
*******************************************************************************/
int Print_Wghts()
{
 int cnt;
 for(cnt=ZERO;cnt<Num_States;cnt++)
  printf("st:%d wght:%f\n",cnt,lstdepth->VA_Re_Wght[cnt]);
 printf("State with max wght:%d wght:%f\n",Max_Wght_St,Max_Wght);
 return(0);
}
/******************************************************************************/
/*                Go backwards and get the last symbol in time.
*******************************************************************************/
int Get_Last_Symbol()
{
 int tstate,tip;
 struct trellis *curr;
 curr=lstdepth;
 tstate=Max_Wght_St;
 /*printf("Backtracking:\n");*/
 while(curr!=NULL)
 {
  tip=curr->ippath[tstate];
  /*printf("currst:%d ip:%d ",tstate,tip);*/
  tstate=curr->stpath[tstate];
  /*printf("Prevst:%d\n",tstate);*/
  curr=curr->prvdepth;
 }
 VA_Op_Sym=tip;                               /* This is the released symbol. */
 /*printf("Released symbol is %d.\n",VA_Op_Sym);*/
 return(0);
}
/******************************************************************************/
/*                        Normalize the metrics.
*******************************************************************************/
int Normalize_Metric()
{
 int cnt;
 Get_Min_Metric();
 Min_Wght=sqrt(Min_Wght);
 for(cnt=ZERO;cnt<Num_States;cnt++)
  Norm_Each_Metric(cnt);
 return(0);
}
/******************************************************************************/
/*                        Get the minimum metric.
*******************************************************************************/
int Get_Min_Metric()
{
 int cnt;
 Min_Wght_St=ZERO;
 Min_Wght=(lstdepth->VA_Re_Wght[ZERO])*(lstdepth->VA_Re_Wght[ZERO]);
 Min_Wght=Min_Wght+(lstdepth->VA_Im_Wght[ZERO])*(lstdepth->VA_Im_Wght[ZERO]);
 for(cnt=ONE;cnt<Num_States;cnt++)
 {
  Cur_Wght=(lstdepth->VA_Re_Wght[cnt])*(lstdepth->VA_Re_Wght[cnt]);
  Cur_Wght=Cur_Wght+(lstdepth->VA_Im_Wght[cnt])*(lstdepth->VA_Im_Wght[cnt]);
  if(Cur_Wght < Min_Wght)
  {
   Min_Wght=Cur_Wght;
   Min_Wght_St=cnt;
  }
 }
 return(0);
}
/******************************************************************************/
/*                Normalize the metric for the current state.
*******************************************************************************/
int Norm_Each_Metric(state)
int state;
{
 Get_Theta(state);
 lstdepth->VA_Re_Wght[state]=lstdepth->VA_Re_Wght[state]-Min_Wght*cos(Theta);
 lstdepth->VA_Im_Wght[state]=lstdepth->VA_Im_Wght[state]-Min_Wght*sin(Theta);
 return(0);
}
/******************************************************************************/
/*                    Get the value of theta mod-2*PI.
*******************************************************************************/
int Get_Theta(state)
int state;
{
 double re,im,cmp;
 re=lstdepth->VA_Re_Wght[state];
 im=lstdepth->VA_Im_Wght[state];
 cmp=im*ERROR;
 if(cmp < FRAC_ZERO)
  cmp= -cmp;
 if((re > cmp) || (re < -cmp))
 {
  Theta=atan(im/re);
  if(re < FRAC_ZERO)
   Theta=Theta+MY_PI;
 }
 else if(im > FRAC_ZERO)
  Theta=PIBY2;
 else
  Theta= -PIBY2;
 return(0);
}
/******************************************************************************/
/*                       Reinitialize Linked List.
*******************************************************************************/
int Linked_List_Reinit()
{
 struct trellis *tnext;
 tnext=fstdepth->nxtdepth;
 tnext->prvdepth=NULL;
 lstdepth->nxtdepth=fstdepth;
 fstdepth->prvdepth=lstdepth;
 fstdepth->nxtdepth=NULL;
 lstdepth=fstdepth;
 fstdepth=tnext;
 return(0);
}
/******************************************************************************/
/*               Reinitialize metrics at the last depth.
*******************************************************************************/
int Metric_Reinit()
{
 int cnt;
 for(cnt=ZERO;cnt<Num_States;cnt++)
 {
  lstdepth->VA_Re_Wght[cnt]= 2*THRESH_WGHT;
  lstdepth->VA_Im_Wght[cnt]= 2*THRESH_WGHT;
 }
 return(0);
}
/******************************************************************************/
