/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "dstype.h"
#include "dsext.h"
/******************************************************************************/
/*                    Allocate memory for the trellis.
*******************************************************************************/
int Trel_Mem_Alloc()
{
 Nxt_State_Alloc();
 M_C_Alloc();
 M_S_Alloc();
 Err_Cnt_Alloc();
 Prv_State_Alloc();
 Re_Wght_Alloc();
 Im_Wght_Alloc();
 return(0);
}
/******************************************************************************/
/*               Allocate memory for storing next states.
*******************************************************************************/
int Nxt_State_Alloc()
{
 int cnt;
 void *tmp;
 void *calloc();
 tmp=calloc(Num_States,sizeof(N_St_Array));
 Ck_Calloc(tmp);
 N_St_Array=tmp;
 for(cnt=ZERO;cnt<Num_States;cnt++)
 {
  tmp=calloc(CONSTELL_SIZE,sizeof(int));
  Ck_Calloc(tmp);
  N_St_Array[cnt]=(int *)tmp;
 }
 return(0);
}
/******************************************************************************/
/* Allocate memory for storing the cosine of the message phase contributions
   corresponding to distinct sampling phases.
*******************************************************************************/
int M_C_Alloc()
{
 int cnt1,cnt2;
 void *tmp;
 void *calloc();
 for(cnt1=ZERO;cnt1<BB_SAMP;cnt1++)
 {
  tmp=calloc(Num_States,sizeof(M_C_Array[cnt1]));
  Ck_Calloc(tmp);
  M_C_Array[cnt1]=tmp;
  for(cnt2=ZERO;cnt2<Num_States;cnt2++)
  {
   tmp=calloc(CONSTELL_SIZE,sizeof(double));
   Ck_Calloc(tmp);
   M_C_Array[cnt1][cnt2]=(double *)tmp;
  }
 }
 return(0);
}
/******************************************************************************/
/* Allocate memory for storing the sine of the message phase contributions
   corresponding to distinct sampling phases.
*******************************************************************************/
int M_S_Alloc()
{
 int cnt1,cnt2;
 void *tmp;
 void *calloc();
 for(cnt1=ZERO;cnt1<BB_SAMP;cnt1++)
 {
  tmp=calloc(Num_States,sizeof(M_S_Array[cnt1]));
  Ck_Calloc(tmp);
  M_S_Array[cnt1]=tmp;
  for(cnt2=ZERO;cnt2<Num_States;cnt2++)
  {
   tmp=calloc(CONSTELL_SIZE,sizeof(double));
   Ck_Calloc(tmp);
   M_S_Array[cnt1][cnt2]=(double *)tmp;
  }
 }
 return(0);
}
/******************************************************************************/
/*  Allocate memory for storing the input count for each state at each depth.
*******************************************************************************/
int Err_Cnt_Alloc()
{
 int cnt;
 void *tmp;
 void *calloc();
 for(cnt=ZERO;cnt<MAX_D_M1;cnt++)
 {
  tmp=calloc(Num_States,sizeof(int));
  Ck_Calloc(tmp);
  Err_Cnt[cnt]=(int *)tmp;
 }
 return(0);
}
/******************************************************************************/
/*             Allocate memory for storing previous states.
*******************************************************************************/
int Prv_State_Alloc()
{
 int cnt;
 void *tmp;
 void *calloc();
 for(cnt=ZERO;cnt<MAX_D_M1;cnt++)
 {
  tmp=calloc(Num_States,sizeof(int));
  Ck_Calloc(tmp);
  Prv_St_Array[cnt]=(int *)tmp;
 }
 return(0);
}
/******************************************************************************/
/*        Allocate memory for storing the real part of the weights.
*******************************************************************************/
int Re_Wght_Alloc()
{
 int cnt;
 void *tmp;
 void *calloc();
 for(cnt=ZERO;cnt<MAX_DEPTH;cnt++)
 {
  tmp=calloc(Num_States,sizeof(double));
  Ck_Calloc(tmp);
  Re_Wght[cnt]=(double *)tmp;
 }
 return(0);
}
/******************************************************************************/
/*        Allocate memory for storing the imaginary part of the weights.
*******************************************************************************/
int Im_Wght_Alloc()
{
 int cnt;
 void *tmp;
 void *calloc();
 for(cnt=ZERO;cnt<MAX_DEPTH;cnt++)
 {
  tmp=calloc(Num_States,sizeof(double));
  Ck_Calloc(tmp);
  Im_Wght[cnt]=(double *)tmp;
 }
 return(0);
}
/******************************************************************************/
/*                 Check for memory allocation failure.
*******************************************************************************/
int Ck_Calloc(tmp)
void *tmp;
{
 if(tmp==NULL)
  Error_Mes(1);
 return(0);
}
/******************************************************************************/
