/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "dstype.h"
#include "dsext.h"
/******************************************************************************/
/*                Search all possible reference sequences.
*******************************************************************************/
int Search_All_Seq()
{
 if(PRN_SEQ_NUM >= Max_Ref_Seq)
  Error_Mes(17);
 Decimal_to_Binary(Ref_Bin_Seq,MAX_D_M1,PRN_SEQ_NUM);
 Get_Reference_Seq();
 Init_Search();
 Dist_Spectrum();
 Sort_Distance();
 Print_Curr_Spectrum();
 return(0);
}
/******************************************************************************/
/*                  This obtains the distance spectrum.
*******************************************************************************/
int Dist_Spectrum()
{
 while(Err_Cnt[ZERO][ZERO] < TWO)
 {
  Nxt_State=N_St_Array[Pre_State][Pre_Input];
  if(Nxt_State == Ref_Nxt_State[Depth])
   Re_Merge_Proc();
  else if(Depth == (MAX_DEPTH-TWO))
   Back_Track();
  else
   Forward_Proc();
 }
 return(0);
}
/******************************************************************************/
/*                           Remerge has occured.
*******************************************************************************/
int Re_Merge_Proc()
{
 printf("Remerge depth:%3d Present state:%3d ",Depth+ONE,Pre_State);
 printf("Next state:%3d Input:%3d.\n",Nxt_State,Pre_Input);
 Update_Metric();
 Store_Distance();
 Back_Track();
 return(0);
}
/******************************************************************************/
/*                           Update the metric.
*******************************************************************************/
int Update_Metric()
{
 int cnt;
 double re_metric,im_metric,re_ref,im_ref,re_err,im_err;
 re_metric=im_metric=0.0;
 for(cnt=ZERO;cnt<BB_SAMP;cnt++)
 {
  re_ref=Ref_C_Op[Depth][cnt];
  im_ref=Ref_S_Op[Depth][cnt];
  re_err=M_C_Array[cnt][Pre_State][Pre_Input];
  im_err=M_S_Array[cnt][Pre_State][Pre_Input];
  re_metric=re_metric+re_ref*re_err+im_ref*im_err;
  im_metric=im_metric+im_ref*re_err-re_ref*im_err;
 } 
 Re_Wght[Depth+ONE][Nxt_State]=Re_Wght[Depth][Pre_State]+re_metric;
 Im_Wght[Depth+ONE][Nxt_State]=Im_Wght[Depth][Pre_State]+im_metric;
 return(0);
}
/******************************************************************************/
/*                    Store the distance in the array.
*******************************************************************************/
int Store_Distance()
{
 /*printf("store metric depth:%d\n",Depth);*/
 if(COHERENT)
  Mag=Re_Wght[Depth+ONE][Nxt_State];
 else
 {
  Sq_Mag=Re_Wght[Depth+ONE][Nxt_State]*Re_Wght[Depth+ONE][Nxt_State];
  Sq_Mag=Sq_Mag+Im_Wght[Depth+ONE][Nxt_State]*Im_Wght[Depth+ONE][Nxt_State];
  Mag=pow(Sq_Mag,0.5);
 }
 Mag=(double)((Depth+ONE)*BB_SAMP)-Mag;
 Search_Store();
 /*printf("inx:%d dist:%f\n",Dist_Store_Inx,Dist_Store[Dist_Store_Inx]);*/
 return(0);
}
/******************************************************************************/
/* See if the current metric is already present in the array. If yes,
   update its multiplicity, else make a new entry in the array.
*******************************************************************************/
int Search_Store()
{
 int cnt,found;
 double diff;
 found=FALSE;
 for(cnt=ZERO;cnt<(Dist_Store_Inx)&&(!found);cnt++)
 {
  diff=Mag-Dist_Store[cnt];
  if((diff < ERROR) && (diff > -ERROR))
  {
   found=TRUE;
   Dist_Mult[cnt]++;
  }
 }
 if(!found)
  Add_New_Entry();
 return(0);
}
/******************************************************************************/
/*                     Add a new entry in the table.
*******************************************************************************/
int Add_New_Entry()
{
 Dist_Mult[Dist_Store_Inx]++;
 Dist_Store[Dist_Store_Inx]=Mag;
 Dist_Store_Inx++;
 if(Dist_Store_Inx == MAX_NUM_DIST)
  Error_Mes(3);
 return(0);
}
/******************************************************************************/
/*                             Backtrack.
*******************************************************************************/
int Back_Track()
{
 Err_Cnt[Depth][Pre_State]++;
 while((Err_Cnt[Depth][Pre_State] == TWO)&&(Depth >= ONE))
 {
  /*printf("backtrack depth:%d pre state:%d ",Depth,Pre_State);*/
  Err_Cnt[Depth][Pre_State]=ZERO;
  Pre_State=Prv_St_Array[Depth][Pre_State];
  /*printf("Prv state:%d\n",Pre_State);*/
  Depth--;
  Err_Cnt[Depth][Pre_State]++;
 }
 Pre_Input=Err_Cnt[Depth][Pre_State]^Ref_Bin_Seq[Depth];
 return(0);
}
/******************************************************************************/
/*                        Proceed to next depth.
*******************************************************************************/
int Forward_Proc()
{
 /*printf("forward depth:%d pst:%d ",Depth,Pre_State);
 printf("inp:%d nst:%d\n",Pre_Input,Nxt_State);*/
 Update_Metric();
 Prv_St_Array[Depth+ONE][Nxt_State]=Pre_State;
 Pre_Input=Err_Cnt[Depth+ONE][Nxt_State]^Ref_Bin_Seq[Depth+ONE];
 Pre_State=Nxt_State;
 Depth++;
 return(0);
}
/******************************************************************************/
/*                       Sort the distance array.
*******************************************************************************/
int Sort_Distance()
{
 int cnt1,cnt2,ulimit;
 ulimit=Dist_Store_Inx-TWO;
 while(ulimit>ZERO)
 {
  for(cnt1=ZERO;cnt1<=ulimit;cnt1++)
  {
   if(Dist_Store[cnt1] > Dist_Store[cnt1+ONE])
    Exchange(cnt1);
  }
  ulimit--;
 }
 return(0);
}
/******************************************************************************/
/*                           Exchange places.
*******************************************************************************/
int Exchange(cnt1)
int cnt1;
{
 int tmp_mult;
 double tmp_dist;
 tmp_dist=Dist_Store[cnt1];
 Dist_Store[cnt1]=Dist_Store[cnt1+1];
 Dist_Store[cnt1+1]=tmp_dist;
 tmp_mult=Dist_Mult[cnt1];
 Dist_Mult[cnt1]=Dist_Mult[cnt1+1];
 Dist_Mult[cnt1+1]=tmp_mult;
 return(0);
}
/******************************************************************************/
/*                       Print the current spectrum.
*******************************************************************************/
int Print_Curr_Spectrum()
{
 Print_Ref();
 Print_Spectrum();
 return(0);
}
/******************************************************************************/
/*                    Print the reference sequence.
*******************************************************************************/
int Print_Ref()
{
 int cnt;
 printf("Reference sequence number is %d.\n",PRN_SEQ_NUM);
 printf("Reference sequence in binary ");
 for(cnt=MAX_D_M1-ONE;cnt>=ZERO;cnt--)
  printf("%d",Ref_Bin_Seq[cnt]);
 printf("*.\n");
 printf("State sequence is ");
 for(cnt=MAX_D_M1-ONE;cnt>=ZERO;cnt--)
  printf("%d,",Ref_Nxt_State[cnt]);
 printf("0*.\n");
 return(0);
}
/******************************************************************************/
/*                           Print the spectrum.
*******************************************************************************/
int Print_Spectrum()
{
 int cnt;
 double zero;
 zero=0.0;
 for(cnt=ZERO;cnt<Dist_Store_Inx;cnt++)
 {
  fprintf(fp2,"%lf %lf\n",Dist_Store[cnt],(double)Dist_Mult[cnt]);
  printf("\t Distance:%12.6f ",Dist_Store[cnt]);
  printf("Multiplicity:%3d\n",Dist_Mult[cnt]);
 }
 printf("\n");
 return(0);
}
/******************************************************************************/
