/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "dstype.h"
#include "dsext.h"
/******************************************************************************/
/*                       Preprocessor functions.
*******************************************************************************/
int Pre_Process()
{
 Check_Input();
 Open_Files();
 Init_Var();
 Get_Tx_Flt();
 Set_Up_Trellis();
 Get_Max_Ref_Seq();
 return(0);
}
/******************************************************************************/
/*                       Check the input parameters.
*******************************************************************************/
int Check_Input()
{
 double curr_mod_fac;
 if(TX_FLT_SIZE%TX_FLT_SAMP)
  Error_Mes(8);
 curr_mod_fac=TWICEPI*MOD_INX/((double)TX_FLT_SAMP*BIT_RATE);
 if(((MOD_FAC-curr_mod_fac)>ERROR) || ((MOD_FAC-curr_mod_fac)< -ERROR))
  Error_Mes(9);
 return(0);
}
/******************************************************************************/
/*                        Open files for graphics.
*******************************************************************************/
int Open_Files()
{
 fp1=fopen("dsdata/txflt.dat","w");
 fp2=fopen("dsdata/dspec.dat","w");
 return(0);
}
/******************************************************************************/
/*                          Initialize variables.
*******************************************************************************/
int Init_Var()
{
 F_s_PB=(double)PB_SAMP*BIT_RATE;
 F_s_BB=(double)BB_SAMP*BIT_RATE;
 F_s_Tx_Flt=(double)TX_FLT_SAMP*BIT_RATE;
 Tx_D_Fac=(int)(TX_FLT_SAMP/PB_SAMP);
 printf("\t Passband sampling frequency is %f kHz.\n\n",F_s_PB);
 printf("\t Baseband sampling frequency is %f kHz.\n\n",F_s_BB);
 printf("\t Transmit filter sampling frequency is %f kHz.\n\n",F_s_Tx_Flt);
 printf("\t Decimation factor at receiver is %d.\n\n",D_FAC);
 printf("\t Decimation factor at transmitter is %d.\n\n",Tx_D_Fac);
 return(0);
}
/******************************************************************************/
/*              Get the transmit-filter (Gaussian) response.
*******************************************************************************/
int Get_Tx_Flt()
{
 Get_Gauss_Flt();
 Conv_Gauss_Rect();
 Normalize_Txflt();
 Print_Tx_Flt();
 return(0);
}
/******************************************************************************/
/*                        Get the Gaussian filter.
*******************************************************************************/
int Get_Gauss_Flt()
{
 int cnt;
 double x,xsq,texpo,delay,index,lnsqrt2,gain;
 lnsqrt2=log(pow(2.0,0.5));
 gain=PI/lnsqrt2;
 gain=BW_3_DB*pow(gain,0.5);
 delay=((double)(G_FLT_SIZE-1))*0.5;
 index=0.0;
 for(cnt=ZERO;cnt<G_FLT_SIZE;cnt++)
 {
  x=BW_3_DB*PI*(index-delay)/F_s_Tx_Flt;
  xsq= -x*x/lnsqrt2;
  texpo=exp(xsq);
  Tx_Flt[cnt]=gain*texpo;
  index=index+1.0;
 }
 return(0);
}
/******************************************************************************/
/*   Convolve Gaussian filter with a rectangular pulse of one bit duration.
*******************************************************************************/
int Conv_Gauss_Rect()
{
 int cnt;
 double allone[TX_FLT_SAMP],op[TX_FLT_SIZE];
 for(cnt=ZERO;cnt<TX_FLT_SAMP;cnt++)
  allone[cnt]=1.0;
 Convolve(allone,Tx_Flt,op,TX_FLT_SAMP,G_FLT_SIZE);
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  Tx_Flt[cnt]=op[cnt];
 return(0);
}
/******************************************************************************/
/*         Normalize the coefficients so that the sum equals 0.5.
*******************************************************************************/
int Normalize_Txflt()
{
 int cnt;
 double sum;
 sum=0.0;
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  sum=sum+Tx_Flt[cnt];
 sum=sum/F_s_Tx_Flt;
 printf("\t Actual area under the Gaussian pulse is %f.\n\n",sum);
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
  Tx_Flt[cnt]=Tx_Flt[cnt]*0.5/sum;
 return(0);
}
/******************************************************************************/
/*           Print the transmit-filter response obtained as above.
*******************************************************************************/
int Print_Tx_Flt()
{
 int cnt;
 double sum;
 sum=0.0;
 for(cnt=ZERO;cnt<TX_FLT_SIZE;cnt++)
 {
  fprintf(fp1,"%f %f\n",(double)cnt,Tx_Flt[cnt]);
  sum=sum+Tx_Flt[cnt];
 }
 sum=sum/F_s_Tx_Flt;
 printf("\t Normalized area under the Gaussian pulse is %f.\n\n",sum);
 return(0);
}
/******************************************************************************/
/*                  Get the maximum input sequence number.
*******************************************************************************/
int Get_Max_Ref_Seq()
{
 int cnt;
 Max_Ref_Seq=ONE;
 for(cnt=ZERO;cnt<MAX_D_M1;cnt++)
  Max_Ref_Seq=Max_Ref_Seq*TWO;
 printf("\t Number of distinct input sequences is %d.\n\n",Max_Ref_Seq);
 return(0);
}
/******************************************************************************/
