/******************************************************************************/
/*   Copyright (C) 1996 K. Vasudevan. Indian Institute of Technology Madras.  */
/*   All Rights Reserved.                                                     */
/******************************************************************************/
/******************************************************************************/
#include "dstype.h"
#include "dsext.h"
/******************************************************************************/
/*                         Set up the trellis.
*******************************************************************************/
int Set_Up_Trellis()
{
 Get_Num_States();
 Trel_Mem_Alloc();
 Get_Trellis();
 return(0);
}
/******************************************************************************/
/*                 Get the number of states in the trellis.
*******************************************************************************/
int Get_Num_States()
{
 int cnt;
 Mem_Size=(int)(TX_FLT_SIZE/TX_FLT_SAMP)-1;
 Num_States=1;
 for(cnt=ZERO;cnt<Mem_Size;cnt++)
  Num_States=2*Num_States;
 Num_States=2*Num_States;                          /* Two extra phase states. */
 printf("\t Memory of the Gaussian filter is %d.\n\n",Mem_Size);
 printf("\t Number of states in the super trellis is %d.\n\n",Num_States);
 return(0);
}
/******************************************************************************/
/* Get the trellis structure. Convention adopted for naming states:
   a) LSB represents the phase. 0:0; 1:PI.
   b) MSB represents latest bit in time. Input bit goes to MSB.
*******************************************************************************/
int Get_Trellis()
{
 int tmp_ipcnt,binarry[BINSIZE];
 for(P_State=ZERO;P_State<Num_States;P_State++)
 {
  Decimal_to_Binary(binarry,BINSIZE,P_State);
  for(P_Input=ZERO;P_Input<TWO;P_Input++)
  {
   for(tmp_ipcnt=ZERO;tmp_ipcnt<BB_SAMP;tmp_ipcnt++)
    Get_M_Phase(tmp_ipcnt,binarry);
   Get_Nxt_State(binarry);
  }
 }
 return(0);
}
/******************************************************************************/
/*     Get the baseband waveform for the sampling phase given by 'tmp_ipcnt'.
*******************************************************************************/
int Get_M_Phase(tmp_ipcnt,array)
int tmp_ipcnt,array[];
{
 M_Phase=0.0;
 Get_Mem_Phase(tmp_ipcnt,array);
 Get_Ip_Phase(tmp_ipcnt);
 Get_State_Phase(array);
 M_C_Array[tmp_ipcnt][P_State][P_Input]=cos(M_Phase);
 M_S_Array[tmp_ipcnt][P_State][P_Input]=sin(M_Phase);
 printf("\t\t Sampling phase:%2d   ",tmp_ipcnt);
 printf("M_Phase:%+12.6f\n\n",M_Phase);
 return(0);
}
/******************************************************************************/
/* Get the phase contributed by the symbols in the memory of the transmit
   filter.
*******************************************************************************/
int Get_Mem_Phase(tmp_ipcnt,array)
int tmp_ipcnt,array[];
{
 int txinx,arinx,ulimit;
 for(arinx=ONE;arinx<=Mem_Size;arinx++)        /* 0th index represents phase. */
 {
  ulimit=TWO*Tx_D_Fac*tmp_ipcnt+(Mem_Size+1-arinx)*TX_FLT_SAMP;
  for(txinx=ZERO;txinx<=ulimit;txinx++)
  {
   if(txinx >= TX_FLT_SIZE)
    Error_Mes(13);
   M_Phase=M_Phase+(double)(TWO*array[arinx])*Tx_Flt[txinx];
  }
 }
 return(0);
}
/******************************************************************************/
/*                Get the phase contributed by the input bit.
*******************************************************************************/
int Get_Ip_Phase(tmp_ipcnt)
int tmp_ipcnt;
{
 int txinx,ulimit;
 ulimit=TWO*Tx_D_Fac*tmp_ipcnt;
 /*printf("mem phase:%12.8lf ph:%d ulim:%d\n\n",M_Phase,tmp_ipcnt,ulimit);*/
 for(txinx=ZERO;txinx<=ulimit;txinx++)
  M_Phase=M_Phase+(double)(TWO*P_Input)*Tx_Flt[txinx];
 M_Phase=M_Phase*MOD_FAC;
 if(M_Phase > TWICEPI)
  M_Phase=M_Phase-TWICEPI;
 return(0);
}
/******************************************************************************/
/*             Get the phase due to the current phase-state.
*******************************************************************************/
int Get_State_Phase(array)
int array[];
{
 M_Phase=M_Phase+PI*(double)array[ZERO];
 if(M_Phase > TWICEPI)
  M_Phase=M_Phase-TWICEPI;
 return(0);
}
/******************************************************************************/
/* Get the next state. This is done as follows:
   a) The phase state is determined by the oldest bit in the transmit filter
      i.e. array[ONE]. If this bit is a ONE, then there is a phase change of
      PI, else there is no phase change. The phase state is given by
      array[ZERO]. This is an XOR operation.
   b) array[ONE] is occupied by the bit in array[TWO] and array[TWO] is
      occupied by the input bit. Input bit moves to MSB.
*******************************************************************************/
int Get_Nxt_State(array)
int array[];
{
 int factor,cnt;
 N_State=(array[ZERO] ^ array[ONE]);
 factor=TWO;
 for(cnt=TWO;cnt<=Mem_Size;cnt++)
 {
  N_State=N_State+factor*array[cnt];
  factor=TWO*factor;
 }
 N_State=N_State+factor*P_Input;
 N_St_Array[P_State][P_Input]=N_State;
 printf("\t Present state:%2d  Input:%2d  ",P_State,P_Input);
 printf("Next state:%2d.\n\n",N_St_Array[P_State][P_Input]);
 return(0);
}
/******************************************************************************/
